package com.savagehomeautomation.devoxx;

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */


import com.google.gson.Gson;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.layout.AnchorPane;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

/**
 * @author Robert Savage 
 * @see http://www.pi4j.com 
 * @see http://www.savagehomeautomation.com
 * @see https://twitter.com/savageautomate
 * 
 * This class is the 'Controller' for the JavaFX scene.
 */
public class Controller implements Initializable {
    
    @FXML private AnchorPane container;
    @FXML private AnchorPane drivewayAlert;
    @FXML private AnchorPane floodAlert;
    @FXML private AnchorPane hvacAlert;
    @FXML private AnchorPane mailboxAlert;
    
    @Override
    public void initialize(URL url, ResourceBundle rb) {

        // ---------------------------------------------------------------------------
        // STARTED
        // ---------------------------------------------------------------------------
        System.out.println(">>> INITIALIZED <<<");
        
        // when starting up, all alerts should be hidden
        drivewayAlert.setVisible(false);
        floodAlert.setVisible(false);
        hvacAlert.setVisible(false);
        mailboxAlert.setVisible(false);

        try {
            // create MQTT client and connect to server
            MqttClient mqtt = MqttHelper.create();
            MqttHelper.connect(mqtt);

            // subscribe to a MQTT topic 
            mqtt.subscribe(MqttHelper.TOPIC);
            
            // set andhandle MQTT callbacks
            mqtt.setCallback(new MqttCallback() {

                @Override
                public void connectionLost(Throwable cause) {
                    System.out.println("[MQTT] --- (CONNECTION LOST) " + cause.getMessage());
                    try {
                        Thread.sleep(1000);
                        MqttHelper.connect(mqtt);                    
                    } catch (InterruptedException | MqttException ex) {
                        Logger.getLogger(Controller.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }

                @Override
                public void messageArrived(String topic, MqttMessage message) throws Exception {
                    System.out.println("[MQTT] <<< (RECEIVED) " + message);
                    try{
                        // attempt to deserialize message payload from MQTT to JSON string and into sensor states object
                        final SensorStates sensorStates = new Gson().fromJson(new String(message.getPayload(), "UTF-8"), SensorStates.class);

                        // run GUI update on UI thread
                        Platform.runLater(() -> {
                            // set the flood alert notification visibility (on message received)
                            floodAlert.setVisible(sensorStates.floodTripped);  
                            hvacAlert.setVisible(sensorStates.hvacTripped);  
                            drivewayAlert.setVisible(sensorStates.drivewayTripped);  
                            mailboxAlert.setVisible(sensorStates.mailboxTripped);  
                        }); 
                    }
                    catch(Exception ex){
                        ex.printStackTrace();
                    }
                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken token) {
                }

            });
        } catch(MqttException | UnknownHostException me) {
            me.printStackTrace();
        }                        
    }        
}